/* Serial LCD support routines */

#define _LCD_c_

#include "project_config.h"

/* local definitions	*/

static BYTE LCDpins;

#define RS	0x04		// RS pin
#define	E	0x08		// E pin

#define SET_RS	LCDpins |= RS		
#define	CLR_RS	LCDpins &= ~RS
#define	SET_E	LCDpins	|= E
#define	CLR_E	LCDpins &= ~E


/* SPI initialization */
/* this routine was borrowed from Microchip peripheral library. It's been rumored that they're going to stop including
	peripherals with new releases of C18 so I make a copy just in case.
	
		sync_mode:
							SPI_FOSC_4 			SPI Master mode, clock = FOSC/4
							SPI_FOSC_16 		SPI Master mode, clock = FOSC/16
							SPI_FOSC_64 		SPI Master mode, clock = FOSC/64
							SPI_FOSC_TMR2 		SPI Master mode, clock = TMR2 output/2
							SLV_SSON 			SPI	Slave mode, /SS pin control enabled
							SLV_SSOFF			SPI	Slave mode, /SS pin control disabled
		bus_mode:
							MODE_00 			SPI bus Mode 0,0
							MODE_01 			SPI bus Mode 0,1
							MODE_10 			SPI bus Mode 1,0
							MODE_11 			SPI bus Mode 1,1
		smp_phase:
							SMPEND 				Input data sample at end of data out
							SMPMID 				Input data sample at middle of data out
*/
void SPI_init( BYTE sync_mode, BYTE bus_mode, BYTE smp_phase )
{
	SSPSTAT &= 0x3f;		//power-on state
	SSPCON1 = 0x00;			//power-on state, SSP disabled
	SSPCON1 |= sync_mode;
	SSPSTAT |= smp_phase;
	
  switch( bus_mode ) {
    case 0:                       // SPI1 bus mode 0,0
      SSPSTATbits.CKE = 1;       // data transmitted on rising edge
      break;    
    case 2:                       // SPI1 bus mode 1,0
      SSPSTATbits.CKE = 1;       // data transmitted on falling edge
      SSPCON1bits.CKP = 1;       // clock idle state high
      break;
    case 3:                       // SPI1 bus mode 1,1
      SSPCON1bits.CKP = 1;       // clock idle state high
      break;
    default:                      // default SPI1 bus mode 0,1
      break;
  }

  switch( sync_mode ) {
    case 4:                       // slave mode w /SS1 enable
      TRISAbits.TRISA5 = 1;       // define /SS1 pin as input
    case 5:                       // slave mode w/o /SS1 enable
      TRISCbits.TRISC3 = 1;       // define clock pin as input
      SSPSTATbits.SMP = 0;       	// must be cleared in slave SPI mode
      break;
    default:                      // master mode, define clock pin as output
      TRISCbits.TRISC3 = 0;       // define clock pin as output
      break;
  }

  TRISC &= 0xDF;                  	// define SDO as output (master or slave)
  TRISC |= 0x10;                  	// define SDI as input (master or slave)
  // IPR1bits.SSPIP = 0;					//low-priority interrupt
  // PIE1bits.SSPIE = 1;					//interrupt enable
  SSPCON1 |= SSPENB;             	// enable synchronous serial port 
}

/* writes to SPI. BF is checked inside the procedure */
/* returns SSPBUF	*/ 
BYTE wr_SPI( BYTE data )
{
	SSPBUF = data;				// write byte to SSP2BUF register
 	while( !SSPSTATbits.BF );	// wait until bus cycle completes
 	return ( SSPBUF );			//
}

/* LCD initialization by instruction 								*/
/* 4-bit 2 line 																		*/
/* wait times are set for 8MHz clock (TCY 500ns)		*/
void LCD_init( void )
{
	CLR_RS;
	RCK = 0;

	/* wait 100msec */
	Delay10KTCYx ( 20 );
	/* send 0x03 */
	LCDpins =  0x30;	// send 0x3
	SET_E;
	SPI_to_74HC595 ();
	CLR_E;
	SPI_to_74HC595 ();
	/* wait 10ms */
	Delay10KTCYx(2);
	SET_E;					// send 0x3
	SPI_to_74HC595 ();
	CLR_E;
	SPI_to_74HC595 ();
	/* wait 10ms */
	Delay10KTCYx(2);
	SET_E;					// send 0x3
	SPI_to_74HC595 ();
	CLR_E;
	SPI_to_74HC595 ();
	/* wait 1ms	*/
	Delay1KTCYx(2);
	LCDpins =  0x20;				// send 0x2 - switch to 4-bit
	SET_E; 
	SPI_to_74HC595();
	CLR_E;
	SPI_to_74HC595();
	/* regular transfers start here */
	Delay1KTCYx(2);
	LCD_sendcmd ( 0x28 );		//4-bit 2-line 5x7-font
	Delay10KTCYx(1);
	LCD_sendcmd ( 0x01 );		//clear display
	Delay10KTCYx(1);
	LCD_sendcmd ( 0x0c );		//turn off cursor, turn on display
	Delay10KTCYx(1);
	LCD_sendcmd ( 0x06 );		//Increrment cursor automatically
	return;
}

/* copies LCDpins variable to parallel output of the shift register */
void SPI_to_74HC595( void )
{
	wr_SPI ( LCDpins );						// send LCDping out the SPI
	RCK = 1;								//move data to parallel pins
	RCK = 0;
}

/* sends a byte to LCD in 4-bit mode */
void LCD_sendbyte( BYTE tosend )
{
	LCDpins &= 0x0f;				//prepare place for the upper nibble 
	LCDpins |= ( tosend & 0xf0 );	//copy upper nibble to LCD variable
	SET_E;							//send
	SPI_to_74HC595();	
	CLR_E;
	SPI_to_74HC595();
	LCDpins &= 0x0f;					//prepare place for the lower nibble
	LCDpins |= (tosend << 4) & 0xf0; 	//copy lower nibble to LCD variable
	SET_E;								//send
	SPI_to_74HC595();	
	CLR_E;
	SPI_to_74HC595();
}

/* send NULL-terminated string */
void LCD_send_string( const rom char *str_ptr )
{
	while (*str_ptr) {
		LCD_sendchar(*str_ptr);
		str_ptr++;
	}
}

/* sends a NULL-terminated RAM string to sendchar() */
void LCD_send_ram_string (char *str_ptr)
{
	while (*str_ptr) {
		LCD_sendchar(*str_ptr);
		str_ptr++;
	}
}

/* sending 2 ASCII symbols representing input byte in hex */
void LCD_send_hexbyte ( BYTE data )
{
	BYTE temp = data>>4;			//prepare first output character

	if ( temp > 9 ) temp+=7;		//jump to letters in ASCII table
	LCD_sendchar ( temp + 0x30 );
	
	data = data & 0x0f;				// mask 4 high bits
	if ( data > 9 ) data+=7;
	LCD_sendchar ( data + 0x30 );	
}

void LCD_second_row( void )
{
	LCD_sendcmd( 0xc0 );
}

void LCD_Home( void )
{
	LCD_sendcmd( LCD_CLRSCR );
	Delay10KTCYx(1);
	LCD_sendcmd( LCD_HOME );
}